%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% StereoSeamCarving.m - Geometrically Consistent Stereo Seam Carving Alg.
% Input:  IL, IR - stereo image pair
%         Disp - disparity map from left to right
%         w - the new width of the retargted images
%         alpha, beta, gamma - weights for the ennergy function: 
%               alpha - the weight of E_{3D} vs. E_{intensity}
%               beta  - the weight of D_n (the normalized dispariy map)] 
%               gamma - the weight of the disparity confidence term
%         (please see  the paper for more details)
% Output: SL, SR - Output left and right seam code images.
%                 Each image SL/SR contains coded seam values 
%                 (SL==1 means first seam to be removed from the left image)
%         Disp - Updated disparity map
%
% Author: Tali Basha (talibasha@gmail.com)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [SL,SR,Disp] = StereoSeamCarving(IL,IR,Disp,w, alpha, beta, gamma, display)

% Initialization
[m, n, c] = size(IL);

 if(c > 1);
        IL_in = rgb2gray(IL);
        IR_in = rgb2gray(IR);
 else
     IL_in = IL;
     IR_in = IR;
 end
 
SL = zeros(m,n);
SR= zeros(m,n);

% computes the occlusions
[Occluder, Occluded] = ComputeOccl(Disp);
% computes the two components of E_{3D}: Eq (8) and D_n
W = ComputeWeights(IL_in, IR_in, Disp, beta, gamma);
% No Match Map- non zero pixels are occluded/occluders pixels
NM_map = Occluder + Occluded;

for i=n:-1:w+1,
    fprintf(1,'Working on seam %d\n',i);        
    [M, SourceMap, UnreachMap] = dynamicProgramming(IL_in,IR_in,Disp,NM_map, W, alpha); 
    x = findOptimalSeam(M,SourceMap);
    if(x==-1)
        return;
    end
    y = mapSeam(SL,x);
    ind = sub2ind([m,n],(1:m)',y);
    SL(ind) = n+1-i;  

     % compute the crossponding 'seam' in the right image 
       SR_temp  = SR;
       ind_seam  = sub2ind([m,n],[1:m]',x);
       xr =  x+round(Disp(ind_seam));
       xr(xr < 1) = 1;
       xr(xr > i-1) = i-1;     
       yr = mapSeam(SR_temp,xr);
       ind = sub2ind([m,n],[1:m]',yr);
       SR_temp(ind) = n+1-i;
       SR = SR_temp;
       % update the disparity values according to the seam 
       Disp = disp_update(Disp, x, xr,Occluded);
       
       if(display)
        Temp = ones(m,n,c);
        Temp(:,1:end-(n-i),:) = IL;
        
        subplot(1,2,1);
        imshow(Temp,[]);  hold on; plot( x,1:m, '.');
        Temp(:,1:end-(n-i),:) = IL;
        subplot(1,2,2);
        imshow(Temp,[]);  hold on; plot( xr,1:m, '.r');
        drawnow;
        
       end
        
       % carve the images
       IL_in = carve(IL_in,x);
       IR_in = carve(IR_in,xr);    
       IL = carve(IL,x);
       IR = carve(IR,xr); 
       
       % carve the occluded and Occluders maps
       Occluded  = carve(Occluded, x); 
       Occluder  = carve(Occluder, x); 
       Disp = carve(Disp,x);
       NM_map = Occluder + Occluded;
       W = carve(W,x);
end

return
