%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% dynamicProgramming.m - Computes a dynamic Programming matrix out of the
% energy matrix.
%
% Input:  IL, IR - Input stereo pair
%         Disp - Disparity map
%         NM_map - occluded/occluding map
%         W - used in the geometry term (E_{3d}) 
%
% Output: M - Dynamic Programming Matrix
%                   UnreachMap - Binary map indicating for each (i,j) if it
%                   unreachable from (i-1, j-1), (i-1,j) and (i-1,j+1)
%                  NearestNonOccl - Indices matrixes that containes for
%                  each unreachble pixels its nearest reachable source.
%
% Author: Tali Basha
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [M, SourceMap, UnreachMap]  = dynamicProgramming(IL,IR, Disp, NM_map, W,alpha)
[m,n] = size(IL);

% foward energy in 3 cases depending on the seam's pixels in the previous
% row, FE1: the previous pixel is to the left, FE2: above, FE3: on the right
FE1 = 0; FE2 = 0; FE3 = 0;

termsV = zeros(3,n-2);
UnreachMap  = zeros(m,n);
SourceMap = zeros(m,n);

% terms 1-3 holds the forward energy for the 3 cases
terms1 = zeros(2, n-2);
terms2 = zeros(1, n-2);
terms3 = zeros(2, n-2);

% normilazed disparity map
NDisp = NormZeroOne(abs(Disp));

% the dynamic prog. matrix
M = zeros(m,n);
    
j = 2:n-1;
M(1,j) = W(1,j) + abs(IL(1,j+1)-IL(1,j-1) ) + alpha*(abs(NDisp(1,j+1)-NDisp(1,j-1))) ; 
M(1,1) = Inf;
M(1,n) = Inf;

for  i=2:m
    j = 1;
    M(i,j) = Inf;
       
    j = 2:n-1;
    termsV(1,:) = M(i-1,j-1) ;
    termsV(2,:) = M(i-1,j  ) ;
    termsV(3,:) = M(i-1,j+1) ;
    
    %  check if the pixel (i,j) is unreachable: if any of the 3 possible
    %  sources are occluded/occluder and the (i,j) is a valid pixel
    ValidPixel = NM_map(i,j)==0;
    
%     unreachable =  ((NM_map(i-1, j-1) > 1 | NM_map(i-1, j)>1 | NM_map(i-1, j+1)>1)) & ValidPixel ;
    unreachable =  (double(NM_map(i-1, j-1) ~= 0) +double( NM_map(i-1, j)~= 0) + double(NM_map(i-1, j+1)~= 0) ==3) & ValidPixel ;
    % update the total unreachable map
    UnreachMap(i, j(unreachable==1) ) = NaN;
    
    terms1(1,:) =   abs(IL(i,j+1)-IL(i,j-1) ) + alpha*(abs(NDisp(i,j+1)-NDisp(i,j-1)));     %horizointal foward energy
    terms1(2,:) =   abs(IL(i-1,j)-IL(i,j-1) ) + alpha*(abs(NDisp(i-1,j)-NDisp(i,j-1)));     %vertical foward energy
    
    terms3(1,:) =  abs(IL(i,j+1)-IL(i,j-1)) +  alpha*(abs(NDisp(i,j+1)-NDisp(i,j-1)));      %horizointal foward energy
    terms3(2,:) =  abs(IL(i-1,j)-IL(i,j+1)) +  alpha*(abs(NDisp(i-1,j)-NDisp(i,j+1)));      %vertical foward energy

    FE1 =  sum(terms1);
    FE3 =  sum(terms3); 
    % in this case there is only horizontal energy
    FE2 =  abs(IL(i,j+1)-IL(i,j-1)) +  alpha*(abs(NDisp(i,j+1)-NDisp(i,j-1)));
    
    % computes the forwoard vertical energy of in the right image
    [E1,E2,E3,HoriznE] =  MVEnergy(Disp(i-1:i,:,:),IR(i-1:i,:,:));
     % adds the energy of the right image
      FE1 = FE1+ E1(j)+HoriznE(j);
      FE2 = FE2+ E2(j)+HoriznE(j);
      FE3 = FE3+ E3(j)+HoriznE(j);
    
        termsV(1,:) =  termsV(1,:) + FE1;
        termsV(2,:) =  termsV(2,:) + FE2;
        termsV(3,:) =  termsV(3,:) + FE3;
    
    M(i,j) =   W(i,j) + min(termsV);
    
    % finds the unreachble pixels indices in the left image
    j_unreach = find(isnan(UnreachMap(i,:)));
    
    % computing the vertical inserted energy in case of generlaized seam in
    % the left image
    if(~isempty(j_unreach))
      [e, j_source]  = VFE_Unreach(Disp(i-1:i, :),IL(i-1:i, :),IR(i-1:i, :), M(i-1,:), j_unreach, 1, alpha, NM_map(i-1,:));
       M(i, j_unreach) =  abs(IL(i,j_unreach+1)-IL(i,j_unreach-1)) +alpha*(abs(NDisp(i,j_unreach+1)-NDisp(i,j_unreach-1))) +1*e;
       SourceMap(i, j_unreach) =   j_source;
    end
        M(i, j(~ValidPixel))  = 9e9;
   
     % Boundary conditions 
    j = n; 
    M(i,j) =   Inf ;
 
end

return

