%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% imretarget.m - Retargets an image given the seam code matrix
%
% Input:  I - Original RGB image to work on
%         S - Seam code matrix
%         w - Number of columns to remove from image I
%
% Output: J - Retargeted image
%
% Retargets image I based on seam image S and the number of columns to be
% removed. If S and I are of the same size, this is straightforward.
% If S is smaller than I, then a special mapping is done between the two
% coordinate systems. In any case w is secified as the number of columns of
% I (and not of S).
%
% Author: Shai Avidan (avidan@adobe.com)
% Date:   Dec 13, 2007
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


function J = imretarget(I,S,w)

if size(I(:,:,1))==size(S)
    J = imageretarget(I,S,w);
else
    J = pyramidretarget(I,S,w);
end

return

function J = imageretarget(I,S,w)

m = size(I,1);
n = size(I,2);
c = size(I,3);
J = zeros(m,n-w,c,class(I));

for j=1:c,
    for i=1:m,
        J(i,:,j) = I(i,S(i,:)>w | S(i,:)==0,j);
    end
end

return

function J = pyramidretarget(I,S,w)

m = size(I,1);
n = size(I,2);
c = size(I,3);
J = zeros(m,n-w,c,class(I));

mi = ceil(log2(min(size(I(:,:,1)))));
ms = ceil(log2(min(size(S))));
levels = mi-ms;
scale = 2^levels;

for k=1:c,
 
    j = 0:n-1;
    jfloor = floor( j / scale );
    jrem   = j - jfloor*scale;
    a      = jfloor+1;
    b      = jrem/scale;
    
    for i=1:m,
        ii = floor((i-1)/scale)+1;
        seam = S(ii,a) + b;
        J(i,:,k) = I(i,seam>(1+(w-1)/scale),k);
    end
end

return

